package edu.harvard.mgh.purcell.gplink.data;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

//import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

//import javax.swing.JOptionPane;

import edu.harvard.mgh.purcell.gplink.mainGUI.*;
import edu.harvard.mgh.purcell.gplink.base_form.*;

/**
 * AutoUpadater extends Timer, automatically updating
 * the output files attached in OpView.
 * @author Kathe Todd-Brown
 *
 */
public class AutoUpdater extends Timer {

	/**
	 * Flags a completed operation
	 */
	public static String COMPLETE = "Complete";
	/**
	 * Flags a running operation
	 */
	public static String RUNNING = "Running";
	/**
	 * Flags a failed operation
	 */
	public static String ERROR = "Failed";
	
	/**
	 * A hash map that keeps all the operations and their current status
	 * completed, running, or failed.
	 */
	public HashMap<String, String> opStatis;
	
	/**
	 * This classes access to the general data
	 */
	private Project data;
	
	/**
	 * This runs every 5 seconds ideal but can be delayed if there isn't
	 * enough processor space.
	 * If the operation is still running then check for a .gplink file
	 * for the completion status. If a running operation is now completed
	 * get the current list of files in our project folder and compares
	 * them to the files listed as operations outputs. If there are files
	 * that are not in the output list but matches the operation root then
	 * it is added to the correct operation output.
	 */
	private TimerTask readLogs = new TimerTask(){
		@SuppressWarnings("static-access")
		public void run() {
			
			//Error out if the operation status is null. This
			//...should never happen.
			if(opStatis == null){
				new ErrorDialog(data.frame, "OpStatus is null.");
				AutoUpdater.this.cancel();
				return;
			}
			
			Vector <String> allFiles = data.getFileList();
			Vector <String> allOps = data.getAllOpName();
			
			if(allOps == null){
				cancel();
				return;
			}
			
			if(data.isRemote()){
				Vector <String> filesToDownload = 
					new Vector<String>();
				
				for(String s: allOps){
					if(s != null
							&& opStatis.get(s) != null
							&& opStatis.get(s).equals(RUNNING)){
						if(allFiles.contains(data.getRemoteFile() 
									+ "/" + s + data.OP_STATUS_EXT)){
							filesToDownload.add(GeneralUtil.makeSpacesHard(data.getRemoteFile() 
									+ "/" + s + data.OP_STATUS_EXT));
						}
						if(allFiles.contains(data.getRemoteFile() 
									+ "/" + s + ".log")){
							filesToDownload.add(GeneralUtil.makeSpacesHard(
										data.getRemoteFile() + "/" + s + ".log"));
						}
					}
				}
				int numdownloads = filesToDownload.size();
				String [] downloadFile = new String [numdownloads];
				for(int i = 0; i < numdownloads; i ++){
					downloadFile[i] = filesToDownload.get(i);
				}
				DownloadFile downloader = 
					new DownloadFile(downloadFile, 
							data.getRoot().getAbsolutePath(), 
							data.getConn(), data.frame);
				downloader.start();
				try { 
					downloader.join();
				} catch (InterruptedException e) {
					new ErrorDialog(data.frame, "Error trying to" +
							" download files.\n run().readLogs@AutoUpdater.java");
				}
			}
			
			boolean change = false;
			
			//Go through each operation and check the status
			for(String s: data.getAllOpName()){
				//Check that the operation is valid and is marked as running
				if(s != null
						&& opStatis.get(s) != null
						&& opStatis.get(s).equals(RUNNING)){
				
					//the file that contanis the status of the operation
					File gplinkLog = new File(data.getRoot(), s + data.OP_STATUS_EXT);
					
					//if this file exist then the command/operation is done
					if(gplinkLog.exists()){
						//initalize the setting to fail
						String setting = "1";
						BufferedReader in;
						try {
							in = new BufferedReader(new FileReader(gplinkLog));
							setting = in.readLine();
							in.close();
						} catch (IOException e) {
							new ErrorDialog(data.frame, "Error trying to read " +
									"in completion data for [ " + s + " ].\n" +
											"Being flagged as unsucessful." +
									"readLogs:AutoUpdater.java");
						}
						
						//0 is a sucessful completion
						if(setting.equals("0")){
							opStatis.put(s, COMPLETE);
							//notify the user
							/*JOptionPane.showMessageDialog(data.frame, 
									s + " is sucessfully completed. \n", 
									"PLINK completed",
				        			JOptionPane.PLAIN_MESSAGE);*/
						} else {
							opStatis.put(s, ERROR);
							/*JOptionPane.showMessageDialog(data.frame, 
									s + " is unsucessfully completed! \n", 
									"PLINK failed",
				        			JOptionPane.ERROR_MESSAGE);*/
						}
						change = true;
					} else
						//the operation is still running
						opStatis.put(s, RUNNING);
				}
			}
			
			if(change == true){
				//update the files associated with the project 
				//...and our displays
				data.frame.updateViews();
			}
			//Saving is good
			data.writeProjectInfo(true, false, false);
			
			////debugging comment to see when we run this
			//Calendar currenttime = Calendar.getInstance();
			//System.out.println("AutoUpdater.java: reading the logs " 
			//		+ DateFormat.getTimeInstance().format(currenttime.getTime()));
			
		}
		
	};
	
	/**
	 * add an operation to our updater and flag it's status correctly.
	 * @param s a String naming the operation
	 */
	@SuppressWarnings("static-access")
	public void addOp(String s){
		File gplinkLog = new File(data.getRoot(), s + data.OP_STATUS_EXT);
		//if the log exists then read in the status and flag it
		if(gplinkLog.exists()){
			BufferedReader in;
			try {
				in = new BufferedReader(new FileReader(gplinkLog));
				String setting = in.readLine();
				in.close();
				if(setting.contains("0"))
					opStatis.put(s, COMPLETE);
				else
					opStatis.put(s, ERROR);
				
			} catch (IOException e) { //you must have a currupted file
				new ErrorDialog(data.frame, "Error reading in completion " +
						"status of [ " + s + "].\n Marking as unsucessful.\n" +
						"addOp:AutoUpdater.java");
				opStatis.put(s, ERROR);
			}
		}
		else
			//flag it as running
			opStatis.put(s, RUNNING);
		
		//update our views
		//data.frame.updateViews();
	}
	
	/**
	 * runs the Timer's cancel and set the opStatis to null for
	 * garbage collection.
	 */
	public void cancel(){
		super.cancel();
		opStatis = null;
	}
	
	public void stop(){
		super.cancel();
	}
	
	/**
	 * Create and start our unique timer. This will update the
	 * status of our operations on the operations viewer.
	 * @param d the current project
	 */
	@SuppressWarnings("static-access")
	public AutoUpdater(Project d) {
		super();
		//System.out.println("Entering AutoUpdater@AutoUpdater.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		///////////////////////////////////
		//*********Initalize***************
		///////////////////////////////////
		data = d;

		opStatis = new HashMap<String, String>();
		
		//go through all the operations in the project
		for (String s : data.getAllOpName()) {
			//look at their status files
			File gplinkLog = new File(data.getRoot(), s + data.OP_STATUS_EXT);
			if(gplinkLog.exists()){
				BufferedReader in;
				try {
					in = new BufferedReader(new FileReader(gplinkLog));
					String setting = in.readLine();
					in.close();
					if(setting.contains("0"))
						opStatis.put(s, COMPLETE);
					else
						opStatis.put(s, ERROR);
				} catch (IOException e){ //you must have a currupted file
					new ErrorDialog(data.frame, "Error reading in completion " +
						"status of [ " + s + "].\n Marking as unsucessful.\n" +
						"addOp:AutoUpdater.java");
				opStatis.put(s, ERROR);
				}
			}
			else
				//the exit status file doesn't exist so we mark it as
				//...running
				opStatis.put(s, RUNNING);
		}
		
		//System.out.println("Flag1 AutoUpdater@AutoUpdater.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		///////////////////////////////////////
		//***********set timer*****************
		////////////////////////////////////////
		readLogs.run();
		
		//System.out.println("Flag2 AutoUpdater@AutoUpdater.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());
		
		long interval = 5*1000;	//time in miliseconds, 5 seconds
		
		//run readLogs every 5 seconds but you can bump it since
		//...it's not critical if the processor is busy
		schedule(readLogs, interval, interval); 
		//System.out.println("Exiting AutoUpdater@AutoUpdater.java "
		//		+ (new GregorianCalendar()).getTimeInMillis());
	}

}
